# pyright: reportMissingTypeStubs = false
# pyright: reportUnknownMemberType = false
import builtins as py_builtins
import traceback
import types
from collections.abc import Callable, Mapping, Sequence
from threading import Event, Thread
from typing import Any, override

from RestrictedPython import (
    Guards,
    limited_builtins,  # pyright: ignore[reportUnknownVariableType]
    safe_builtins,  # pyright: ignore[reportUnknownVariableType]
)

from _aegis_game.types import MethodDict

from .sandbox import Sandbox


class LumenCore:
    """Core executor for running agent code in a restricted, sandboxed environment."""

    def __init__(
        self, code: Sandbox, methods: MethodDict, error: Callable[..., None]
    ) -> None:
        """
        Initialize the LumenCore executor.

        Args:
            code: A sandboxed script containing the agent logic.
            methods: A dictionary of allowed API methods for the agent.
            error: A callback to report errors during execution.

        """
        self.code: Sandbox = code
        self.methods: MethodDict = methods
        self.error: Callable[..., None] = error
        self.initialized: bool = False
        self.thread: LumenThread = LumenThread(self)
        self.thread.start()

        self.allowed_modules: set[str] = {
            "os",
            "typing",
            "pathlib",
            "random",
            "heapq",
            "math",
            "json",
            "re",
            "enum",
            "numpy",
            "tensorflow",
            "tf",
        }

        self.namespace: dict[str, object] = self._build_namespace()

    def _build_namespace(self) -> dict[str, object]:
        """
        Construct the restricted execution namespace for the agent.

        Returns:
            A dictionary representing the sandboxed execution environment.

        """
        builtins: MethodDict = {
            **safe_builtins,
            **limited_builtins,
            **self.methods,
            **{
                name: getattr(py_builtins, name)
                for name in [
                    "all",
                    "any",
                    "list",
                    "dict",
                    "set",
                    "tuple",
                    "enumerate",
                    "reversed",
                    "max",
                    "min",
                    "sum",
                ]
            },
        }

        builtins["__import__"] = self.custom_import

        namespace: dict[str, object] = {
            "__builtins__": builtins,
            "__name__": "__main__",
            "_getattr_": self.deny_private_attr,
            "_getitem_": self.deny_private_items,
            "_getiter_": self.default_guarded_iter,
            "_write_": self.default_guarded_write,
            "__metaclass__": type,
            "_unpack_sequence_": Guards.guarded_unpack_sequence,
            "_iter_unpack_sequence_": Guards.guarded_iter_unpack_sequence,
        }

        return namespace

    def default_guarded_iter(self, ob: object) -> object:
        """Bypass iteration restrictions (safe override for RestrictedPython)."""
        return ob

    def default_guarded_write(self, ob: object) -> object:
        """Bypass write restrictions (safe override for RestrictedPython)."""
        return ob

    def custom_import(
        self,
        name: str,
        globals_: Mapping[str, object] | None = None,
        locals_: Mapping[str, object] | None = None,
        fromlist: Sequence[str] = (),
        level: int = 0,
    ) -> types.ModuleType:
        """
        Import guard for controlling allowed modules.

        Disallows relative/private imports and validates modules against allowlist.
        """
        # Allow "import module" if it's in the allowed list
        if not fromlist and name not in self.allowed_modules:
            error = f"Import of module '{name}' is not allowed"
            raise ImportError(error)

        # Invalid imports
        if fromlist and not isinstance(fromlist, tuple):
            error = f"Invalid Import: {name}"
            raise ImportError(error)

        # Disallow relative imports
        if level != 0:
            error = f"Relative imports not allowed: {name}"
            raise ImportError(error)

        # Disallow private imports
        if name.startswith("_") or name not in self.allowed_modules:
            error = f"Import of module '{name}' is not allowed"
            raise ImportError(error)
        return __import__(name, globals_, locals_, fromlist, level)  # pyright: ignore[reportAny]

    @staticmethod
    def deny_private_attr(obj: object, attr: str) -> object:
        """Deny access to private attributes."""
        if attr.startswith("_"):
            error = f"Access to private attribute '{attr}' is denied."
            raise AttributeError(error)
        return getattr(obj, attr)  # pyright: ignore[reportAny]

    @staticmethod
    def deny_private_items(obj: Mapping[Any, Any], item: object) -> object:  # pyright: ignore[reportExplicitAny]
        """Deny access to private keys."""
        if isinstance(item, str) and item.startswith("_"):
            error = f"Access to private key '{item}' is denied."
            raise KeyError(error)
        return obj[item]  # pyright: ignore[reportAny]

    def init(self) -> None:
        """Initialize the agent's main code."""
        try:
            exec(self.code["main"], self.namespace)  # noqa: S102
            self.initialized = True
        except Exception:  # noqa: BLE001
            self.error(traceback.format_exc(limit=5))

    def think(self) -> None:
        """Call the agent's `think()` function. Reports errors if not defined or fails."""
        fn = self.namespace.get("think")
        if not callable(fn):
            self.error("Think doesn't exist")
            return
        try:
            _ = fn()
        except Exception:  # noqa: BLE001
            self.error(traceback.format_exc(limit=5))

    def run(self) -> None:
        """Triggers a turn execution in the LumenThread and waits for completion."""
        self.thread.trigger_turn()
        self.thread.wait_for_turn()

    def kill(self) -> None:
        """Terminates the thread execution and waits for cleanup."""
        self.thread.kill()
        self.thread.join()


class LumenThread(Thread):
    """Manages a background thread for executing agent logic turn-by-turn."""

    def __init__(self, runner: LumenCore) -> None:
        """
        Initialize a turn-based thread runner.

        Args:
            runner: The LumenCore instance this thread manages.

        """
        super().__init__()
        self.runner: LumenCore = runner
        self.running: bool = True
        self.paused: bool = False

        self.run_event: Event = Event()
        self.pause_event: Event = Event()
        self.turn_done_event: Event = Event()

    @override
    def run(self) -> None:
        """Thread loop. Waits for turn triggers and runs agent logic."""
        while self.running:
            _ = self.run_event.wait()
            if not self.running:
                break

            if not self.runner.initialized:
                self.runner.init()

            self.runner.think()
            self.run_event.clear()
            self.turn_done_event.set()

    def trigger_turn(self) -> None:
        """Signals the thread to start or resume a turn."""
        if self.paused:
            self.pause_event.set()
        else:
            self.run_event.set()

    def wait_for_turn(self) -> None:
        """Wait for the current turn to finish."""
        _ = self.turn_done_event.wait()
        _ = self.turn_done_event.clear()

    def wait_for_resume(self) -> None:
        """Wait for the thread to resume if paused."""
        self.turn_done_event.set()
        _ = self.pause_event.wait()
        self.pause_event.clear()

    def kill(self) -> None:
        """Gracefully shuts down the thread."""
        self.running = False
        self.run_event.set()
        self.pause_event.set()



# noqa: INP001

"""Fix protobuf Python imports in generated files and fix index.ts export."""

import re
from itertools import chain
from pathlib import Path

PYTHON_DIR = Path(__file__).parent.parent / "python"


def fix_imports(file: Path) -> None:
    """Fix the broken imports caused by protoc Python plugin."""
    content = file.read_text()

    # Replace 'import xyz_pb2 as _xyz_pb2' with 'from . import xyz_pb2 as _xyz_pb2'
    pattern = re.compile(r"^import (.+_pb2) as (.+_pb2)", re.MULTILINE)
    fixed_content = pattern.sub(r"from . import \1 as \2", content)

    _ = file.write_text(fixed_content)


def main() -> None:
    """Run all fixes."""
    print(f"Fixing imports in {PYTHON_DIR}")
    for py_file in chain(PYTHON_DIR.glob("*.py"), PYTHON_DIR.glob("*.pyi")):
        print(f"Fixing {py_file.name}")
        fix_imports(py_file)
    print("Done fixing files.")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: turn.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'turn.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import location_pb2 as location__pb2
from . import spawn_pb2 as spawn__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nturn.proto\x12\x05\x61\x65gis\x1a\x0elocation.proto\x1a\x0bspawn.proto\"\x90\x01\n\x04Turn\x12\x0f\n\x07\x61gentId\x18\x01 \x01(\x05\x12\x14\n\x0c\x65nergy_level\x18\x02 \x01(\x05\x12\x13\n\x0bsteps_taken\x18\x03 \x01(\x05\x12\x1c\n\x03loc\x18\x04 \x01(\x0b\x32\x0f.aegis.Location\x12\x10\n\x08\x63ommands\x18\x05 \x03(\t\x12\x1c\n\x06spawns\x18\x06 \x03(\x0b\x32\x0c.aegis.Spawnb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'turn_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_TURN']._serialized_start=51
  _globals['_TURN']._serialized_end=195
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from enum import Enum
from typing import override

from .world_object import WorldObject


class Survivor(WorldObject):
    """
    Represents a survivor in the world.

    Attributes:
        id: Unique identifier for the survivor.
        health: Current health of the survivor.

    """

    def __init__(
        self, survivor_id: int = -1, health: int = 1, state: State | None = None
    ) -> None:
        super().__init__()
        self._state: Survivor.State = state if state is not None else self.State.ALIVE
        self.id: int = survivor_id
        self.health: int = health

    def is_alive(self) -> bool:
        """
        Check if the survivor is alive.

        Returns:
            True if the survivor is ALIVE, False otherwise.

        """
        return self._state == self.State.ALIVE

    def set_state(self, state: State) -> None:
        """
        Set the survivor's state.

        Args:
            state: The new state for the survivor.

        """
        self._state = state

    def get_state(self) -> State:
        """
        Get the survivor's current state.

        Returns:
            The current state of the survivor.

        """
        return self._state

    @override
    def __str__(self) -> str:
        return f"SURVIVOR ( ID {self.id} , HP {self.health} )"

    @override
    def __repr__(self) -> str:
        return self.__str__()

    class State(Enum):
        ALIVE = 0
        DEAD = 1



from . import location_pb2 as _location_pb2
from . import spawn_pb2 as _spawn_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Turn(_message.Message):
    __slots__ = ("agentId", "energy_level", "steps_taken", "loc", "commands", "spawns")
    AGENTID_FIELD_NUMBER: _ClassVar[int]
    ENERGY_LEVEL_FIELD_NUMBER: _ClassVar[int]
    STEPS_TAKEN_FIELD_NUMBER: _ClassVar[int]
    LOC_FIELD_NUMBER: _ClassVar[int]
    COMMANDS_FIELD_NUMBER: _ClassVar[int]
    SPAWNS_FIELD_NUMBER: _ClassVar[int]
    agentId: int
    energy_level: int
    steps_taken: int
    loc: _location_pb2.Location
    commands: _containers.RepeatedScalarFieldContainer[str]
    spawns: _containers.RepeatedCompositeFieldContainer[_spawn_pb2.Spawn]
    def __init__(self, agentId: _Optional[int] = ..., energy_level: _Optional[int] = ..., steps_taken: _Optional[int] = ..., loc: _Optional[_Union[_location_pb2.Location, _Mapping]] = ..., commands: _Optional[_Iterable[str]] = ..., spawns: _Optional[_Iterable[_Union[_spawn_pb2.Spawn, _Mapping]]] = ...) -> None: ...



import os
import sys

from src.agents.better_agent.model import Model

sys.path.insert(0, os.getcwd())
import random

import numpy as np

curr_dir = os.path.dirname(os.path.realpath(__file__))
os.environ["PYTHONPATH"] = os.path.join(curr_dir, "src")


def main():
    random.seed(12345)
    np.random.seed(12345)

    model = Model()
    trained_model = model.get_model()  # load and train a model

    _, _, x_test, y_test = model._load_data()

    num_samples_to_test = 10
    correct_predictions = 0

    for _ in range(num_samples_to_test):
        index = random.randint(0, len(x_test) - 1)

        image = x_test[index]
        label = y_test[index]
        image = np.expand_dims(image, axis=0)

        print(f"Image shape: {image.shape}")
        print(f"Image dtype: {image.dtype}")

        predictions = trained_model.predict(image, verbose=0)
        predicted_label = np.argmax(predictions[0])

        is_correct = label == predicted_label
        correct_predictions += is_correct

        print(
            f"True Label: {label}, Predicted Label: {predicted_label}, Correct: {is_correct}"
        )

        # plt.imshow(x_test[index], cmap="gray")
        # plt.title(f"Label: {y_test[index]}")
        # plt.show()

    accuracy = correct_predictions / num_samples_to_test
    print(f"Accuracy for {num_samples_to_test} samples: {accuracy * 100:.2f}%")


if __name__ == "__main__":
    curr_dir = os.path.dirname(os.path.realpath(__file__))
    os.environ["PYTHONPATH"] = os.path.join(curr_dir, "src")
    main()



from dataclasses import dataclass
from enum import Enum


@dataclass(frozen=True)
class Attributes:
    energy_multiplier: float
    action_cooldown: int


class AgentType(Enum):
    MEDIC = Attributes(0.5, 10)
    """Medic type."""
    ENGINEER = Attributes(0.75, 10)
    """Engineer type."""
    COMMANDER = Attributes(1.0, 10)
    """Commander type. Also used when the config `ALLOW_AGENT_TYPES` is disabled."""

    @property
    def energy_multiplier(self) -> float:
        """Multiplier applied to the world's start energy to determine the agent's energy."""
        return self.value.energy_multiplier

    @property
    def action_cooldown(self) -> int:
        """Action cooldown of the agent."""
        return self.value.action_cooldown



# pyright: reportMissingTypeStubs = false
from __future__ import annotations

import logging
import re
from typing import TYPE_CHECKING, cast

from RestrictedPython.compile import (
    compile_restricted,  # pyright: ignore[reportUnknownVariableType]
)

from _aegis_game.logger import LOGGER

from .typed_ast import NodeTransformer

if TYPE_CHECKING:
    from pathlib import Path
    from types import CodeType

logger = logging.getLogger(__name__)


class Sandbox:
    def __init__(self, code: dict[str, CodeType]) -> None:
        """
        Initialize CodeSandbox with compiled code modules.

        Args:
            code: Dictionary mapping module names to compiled code objects.

        """
        self.code: dict[str, CodeType] = code.copy()

    @classmethod
    def from_directory_dict(cls, files: dict[str, str]) -> Sandbox:
        """
        Create CodeSandbox from a dictionary of filename -> source code mappings.

        Args:
            files: Dictionary mapping filenames to their source code content.

        Returns:
            A new CodeSandbox instance with compiled modules.

        Raises:
            CompilationError: If any file fails to compile.

        """
        code: dict[str, CodeType] = {}

        for filename, source in files.items():
            if not filename.endswith(".py"):
                LOGGER.warning(f"Skipping non-Python file: {filename}")
                continue

            module_name = filename.removesuffix(".py")
            cleaned_source = cls._strip_stub_import(source)

            try:
                compiled = cast(
                    "CodeType",
                    compile_restricted(
                        cleaned_source,
                        filename,
                        "exec",
                        policy=NodeTransformer,
                    ),
                )
            except Exception as e:
                error = f"Failed to compile {filename}: {e}"
                raise CompilationError(error) from e

            code[module_name] = compiled

        return cls(code)

    @classmethod
    def from_directory(cls, directory_path: Path) -> Sandbox:
        """
        Create CodeSandbox from a directory containing Python files.

        Args:
            directory_path: Path to directory containing Python files.

        Returns:
            A new CodeSandbox instance with compiled modules from the directory.

        Raises:
            CodeSandboxError: If directory doesn't exist or is inaccessible.
            CompilationError: If any file fails to compile.

        """
        if not directory_path.exists():
            error = f"Directory does not exist: {directory_path}"
            raise SandboxError(error)

        if not directory_path.is_dir():
            error = f"Path is not a directory: {directory_path}"
            raise SandboxError(error)

        try:
            files: dict[str, str] = {}
            for file_path in directory_path.glob("*.py"):
                if file_path.is_file():
                    try:
                        files[file_path.name] = file_path.read_text(encoding="utf-8")
                    except (OSError, UnicodeDecodeError) as e:
                        LOGGER.warning(f"Failed to read {file_path}: {e}")
                        continue

            if not files:
                LOGGER.warning(f"No readable Python files found in {directory_path}")

            return cls.from_directory_dict(files)

        except OSError as e:
            error = f"Failed to access directory {directory_path}: {e}"
            raise SandboxError(error) from e

    @staticmethod
    def _strip_stub_import(code: str) -> str:
        """
        Remove aegis_game.stub import statements from code.

        Args:
            code: Source code to process.

        Returns:
            Code with aegis_game.stub imports removed.

        """
        pattern = r"^.*from\s+aegis_game\.stub\s+import.*(?:\r\n|\r|\n)?"
        return re.sub(pattern, "", code, flags=re.MULTILINE)

    def __getitem__(self, module_name: str) -> CodeType:
        """Get the compiled module code by module name."""
        return self.code[module_name]

    def __contains__(self, module_name: str) -> bool:
        """Check if module is in the sandbox using `in`."""
        return module_name in self.code


class SandboxError(Exception):
    """Base exception for CodeSandbox operations."""


class CompilationError(SandboxError):
    """Raised when code compilation fails."""



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import override


class WorldObject(ABC):
    def __init__(self) -> None:
        self.id: int = -1

    @abstractmethod
    @override
    def __str__(self) -> str:
        pass



#!/usr/bin/env python3
"""
Script to check protobuf version compatibility.
Ensures the generated protobuf files are compatible with the installed runtime.
"""

import re
import sys
from pathlib import Path


def extract_protobuf_version_from_file(file_path: str) -> str:
    """Extract protobuf version from a generated protobuf file."""
    try:
        with open(file_path, "r", encoding="utf-8") as f:
            content = f.read()
            # Look for the protobuf version comment
            match = re.search(r"# Protobuf Python Version: (\d+\.\d+\.\d+)", content)
            if match:
                return match.group(1)
    except Exception as e:
        print(f"Error reading {file_path}: {e}")
    return None


def check_protobuf_versions():
    """Check protobuf versions in generated files."""
    print("Checking protobuf version compatibility...")

    # Check Python files
    python_dir = Path("python")
    if python_dir.exists():
        print("\nPython protobuf files:")
        versions = set()
        for py_file in python_dir.glob("*.py"):
            version = extract_protobuf_version_from_file(str(py_file))
            if version:
                versions.add(version)
                print(f"  {py_file.name}: {version}")

        if len(versions) > 1:
            print(f"\n⚠️  WARNING: Multiple protobuf versions found: {versions}")
            print("   This may cause compatibility issues.")
        elif versions:
            print(f"\n✅ All Python files use protobuf version: {list(versions)[0]}")

    # Check TypeScript files (they don't have version comments, but we can verify they exist)
    ts_dir = Path("ts")
    if ts_dir.exists():
        ts_files = list(ts_dir.glob("*.ts"))
        print(f"\nTypeScript protobuf files: {len(ts_files)} files found")

    print("\nProtobuf version check completed!")


if __name__ == "__main__":
    check_protobuf_versions()



# pyright: reportUnknownMemberType=false
from pathlib import Path
from typing import TYPE_CHECKING, cast

import numpy as np
import tensorflow as tf

if TYPE_CHECKING:
    from numpy.typing import NDArray


def train_model() -> None:
    """Train and save a simple placeholder symbol prediction model."""
    # Load training data
    data_dir: Path = (
        Path(__file__).parent.parent.parent / "prediction_data" / "training"
    )
    x_train: NDArray[np.uint8] = cast(
        "NDArray[np.uint8]", np.load(data_dir / "x_train_symbols.npy")
    )
    y_train: NDArray[np.int32] = cast(
        "NDArray[np.int32]", np.load(data_dir / "y_train_symbols.npy")
    )
    print(type(x_train))
    print(type(y_train))
    print(x_train.shape)
    print(y_train.shape)
    print(x_train.dtype)
    print(y_train.dtype)

    # Normalize the data
    x_train_normalized: NDArray[np.float32] = x_train.astype(np.float32) / 255.0

    # Convert labels to categorical
    num_classes: int = len(np.unique(y_train))
    y_train_categorical: NDArray[np.float32] = tf.keras.utils.to_categorical(
        y_train, num_classes
    )

    # Create model
    model: tf.keras.Sequential = tf.keras.Sequential(
        [
            tf.keras.layers.Flatten(input_shape=(28, 28)),
            tf.keras.layers.Dense(128, activation="relu"),
            tf.keras.layers.Dense(num_classes, activation="softmax"),
        ]
    )

    # Compile model
    model.compile(
        optimizer="adam", loss="categorical_crossentropy", metrics=["accuracy"]
    )

    # Train model
    model.fit(
        x_train_normalized,
        y_train_categorical,
        epochs=10,
        batch_size=32,
        validation_split=0.2,
    )

    # Save model
    model.save("trained_net.keras")


if __name__ == "__main__":
    train_model()



from __future__ import annotations

from enum import Enum


class Direction(Enum):
    """
    Represents the eight principal compass directions plus the center (no movement).

    Each direction has a tuple value representing its (dx, dy) movement on a grid.
    """

    NORTH = (0, 1)
    """Direction that points north (up)."""

    NORTHEAST = (1, 1)
    """Direction that points northeast (up and right)."""

    EAST = (1, 0)
    """Direction that points east (right)."""

    SOUTHEAST = (1, -1)
    """Direction that points southeast (down and right)."""

    SOUTH = (0, -1)
    """Direction that points south (down)."""

    SOUTHWEST = (-1, -1)
    """Direction that points southwest (down and left)."""

    WEST = (-1, 0)
    """Direction that points west (left)."""

    NORTHWEST = (-1, 1)
    """Direction that points northwest (up and left)."""

    CENTER = (0, 0)
    """Direction that points center (not moving)."""

    @property
    def dx(self) -> int:
        """The change in the x direction."""
        return self.value[0]

    @property
    def dy(self) -> int:
        """The change in the y direction."""
        return self.value[1]

    def rotate_left(self) -> Direction:
        """
        Rotate the direction 45 degrees counter-clockwise (left).

        The center direction returns itself unchanged.

        Returns:
            The direction rotated left.

        """
        if self == Direction.CENTER:
            return self
        new_index = (dir_to_index[self] - 1) % 8
        return dir_order[new_index]

    def rotate_right(self) -> Direction:
        """
        Rotate the direction 45 degrees clockwise (right).

        The center direction returns itself unchanged.

        Returns:
            The direction rotated right.

        """
        if self == Direction.CENTER:
            return self
        new_index = (dir_to_index[self] + 1) % 8
        return dir_order[new_index]

    def get_opposite(self) -> Direction:
        """
        Get the opposite direction (180 degrees rotation).

        The center direction returns itself unchanged.

        Returns:
            The opposite direction.

        """
        if self == Direction.CENTER:
            return self
        new_index = (dir_to_index[self] + 4) % 8
        return dir_order[new_index]


dir_order = [
    Direction.NORTH,
    Direction.NORTHEAST,
    Direction.EAST,
    Direction.SOUTHEAST,
    Direction.SOUTH,
    Direction.SOUTHWEST,
    Direction.WEST,
    Direction.NORTHWEST,
    Direction.CENTER,
]
dir_to_index = {d: i for i, d in enumerate(dir_order)}



from __future__ import annotations

from typing import override

from _aegis_game.common.direction import Direction


class Location:
    """
    Represents a coordinate location with x and y integer values.

    Attributes:
        x: The x-coordinate.
        y: The y-coordinate.

    """

    def __init__(self, x: int, y: int) -> None:
        """
        Initialize a Location with given x and y coordinates.

        Args:
            x: The x-coordinate.
            y: The y-coordinate.

        """
        self.x: int = x
        self.y: int = y

    def add(self, direction: Direction) -> Location:
        """
        Return a new location by moving this Location in the specified direction.

        Args:
            direction: The direction to move.

        Returns:
            A new location in the given direction.

        """
        return Location(self.x + direction.dx, self.y + direction.dy)

    def direction_to(self, location: Location) -> Direction:
        """
        Determine the cardinal direction from this location to another location.

        Args:
            location: The target location.

        Returns:
            The cardinal direction pointing towards the target location.

        """
        dx = location.x - self.x
        dy = location.y - self.y

        key = (dx > 0) - (dx < 0), (dy > 0) - (dy < 0)

        direction_map = {
            (1, 1): Direction.NORTHEAST,
            (1, -1): Direction.SOUTHEAST,
            (-1, 1): Direction.NORTHWEST,
            (-1, -1): Direction.SOUTHWEST,
            (1, 0): Direction.EAST,
            (-1, 0): Direction.WEST,
            (0, 1): Direction.NORTH,
            (0, -1): Direction.SOUTH,
            (0, 0): Direction.CENTER,
        }

        return direction_map.get(key, Direction.CENTER)

    def distance_to(self, location: Location) -> int:
        """
        Calculate the squared Euclidean distance from this location to another.

        Args:
            location: The target location.

        Returns:
            int: The squared distance between the two locations.

        """
        dx = self.x - location.x
        dy = self.y - location.y
        return dx * dx + dy * dy

    def is_adjacent_to(self, location: Location) -> bool:
        """
        Check if this Location is adjacent to another Location.

        Arguments:
            location: The target location.

        Returns:
            True if the other location is adjacent, else False.

        """
        dx = self.x - location.x
        dy = self.y - location.y
        return -1 <= dx <= 1 and -1 <= dy <= 1

    @override
    def __str__(self) -> str:
        return f"({self.x}, {self.y})"

    @override
    def __repr__(self) -> str:
        return self.__str__()

    @override
    def __hash__(self) -> int:
        value = 3
        value = 89 * value + self.x
        return 89 * value + self.y

    @override
    def __eq__(self, other: object) -> bool:
        if isinstance(other, Location):
            return self.x == other.x and self.y == other.y
        return False

    @override
    def __ne__(self, other: object) -> bool:
        if isinstance(other, Location):
            return not self.__eq__(other)
        return False

    def __lt__(self, other: object) -> bool:
        if isinstance(other, Location):
            if self.x < other.x:
                return True
            if self.x == other.x:
                return self.y < other.y
        return False

    def __gt__(self, other: object) -> bool:
        if isinstance(other, Location):
            if self.x > other.x:
                return True
            if self.x == other.x:
                return self.y > other.y
        return False

    def __le__(self, other: object) -> bool:
        if isinstance(other, Location):
            return self.__lt__(other) or self.__eq__(other)
        return False

    def __ge__(self, other: object) -> bool:
        if isinstance(other, Location):
            return self.__gt__(other) or self.__eq__(other)
        return False



from typing import override


class Message:
    """
    Represents a message sent by an agent during a specific round.

    Attributes:
        message: The content of the message.
        round_num: The round number when the message was sent.
        sender_id: The ID of the agent who sent the message.

    """

    def __init__(self, message: str, round_num: int, sender_id: int) -> None:
        self.message: str = message
        self.round_num: int = round_num
        self.sender_id: int = sender_id

    @override
    def __str__(self) -> str:
        return f'Msg from Agent#{self.sender_id} at round {self.round_num}: "{self.message}"'

    @override
    def __repr__(self) -> str:
        return self.__str__()



# pyright: reportImportCycles = false

from typing import TYPE_CHECKING

from .agent_type import AgentType
from .common import Direction, Location
from .constants import Constants
from .logger import AGENT_LOGGER
from .message_buffer import MessageBuffer
from .sandbox.core import LumenCore
from .sandbox.sandbox import Sandbox
from .team import Team
from .types import MethodDict

if TYPE_CHECKING:
    from .game import Game


class Agent:
    def __init__(  # noqa: PLR0913
        self,
        game: "Game",
        agent_id: int,
        location: Location,
        team: Team,
        energy_level: int,
        agent_type: AgentType,
    ) -> None:
        self.game: Game = game
        self.has_visited: list[bool] = [False] * (game.world.height * game.world.width)
        self.id: int = agent_id
        self.team: Team = team
        self.location: Location = location
        self.energy_level: int = energy_level
        self.type: AgentType = agent_type
        self.action_cooldown: int = agent_type.action_cooldown
        self.core: LumenCore | None = None
        self.message_buffer: MessageBuffer = MessageBuffer()
        self.steps_taken: int = 0
        self.debug: bool = False
        self.errors: list[str] = []

    def process_beginning_of_turn(self) -> None:
        if self.core is None:
            error = "Trying to run an agent that hasn't launched"
            raise RuntimeError(error)

        self.action_cooldown = max(0, self.action_cooldown - Constants.COOLDOWN_TICK)

    def process_end_of_turn(self) -> None:
        self.game.game_pb.end_turn(self)

    def turn(self) -> None:
        self.process_beginning_of_turn()
        self.errors.clear()
        self.core.run()  # pyright: ignore[reportOptionalMemberAccess]
        self.log_errors()
        self.penalize_for_errors()
        self.process_end_of_turn()

    def kill(self) -> None:
        self.core.kill()  # pyright: ignore[reportOptionalMemberAccess]

    def launch(
        self, code: Sandbox | None, methods: MethodDict, *, debug: bool = False
    ) -> None:
        if code is None:
            error = "No code provided to launch."
            raise ValueError(error)

        self.core = LumenCore(code, methods, self.error)
        self.debug = debug

    def apply_movement_cost(self, direction: Direction) -> None:
        if direction == Direction.CENTER:
            return

        cell = self.game.get_cell_at(self.location.add(direction))
        self.add_energy(-cell.move_cost)
        self.steps_taken += 1

    def add_energy(self, energy: int) -> None:
        self.energy_level += energy
        self.energy_level = min(Constants.MAX_ENERGY_LEVEL, self.energy_level)

    def add_cooldown(self, cooldown: int = -1) -> None:
        self.action_cooldown = self.type.action_cooldown if cooldown == -1 else cooldown

    def error(self, msg: str) -> None:
        self.errors.append(msg)

    def log_errors(self) -> None:
        for error in self.errors:
            if self.debug:
                self.log(error, is_error=True)
            else:
                AGENT_LOGGER.warning(
                    f"[Agent#({self.id}:{self.team.name})@{self.game.round}] [ERROR] Error thrown this round. (Turn on debug to see error message)"
                )

    def log(self, *args: object, is_error: bool = False) -> None:
        if not self.debug:
            return

        agent_id = self.id
        prefix = f"[Agent#({agent_id}:{self.team.name})@{self.game.round}]"

        if is_error:
            AGENT_LOGGER.error(f"{prefix} {' '.join(map(str, args))}")
        else:
            AGENT_LOGGER.info(f"{prefix} {' '.join(map(str, args))}")

    def penalize_for_errors(self) -> None:
        if self.errors:
            self.add_energy(Constants.ENERGY_PENALTY_FOR_ERRORS)
            AGENT_LOGGER.warning(
                f"[Agent#({self.id}:{self.team.name})@{self.game.round}] [ERROR] penalized {Constants.ENERGY_PENALTY_FOR_ERRORS} energy for error."
            )



import random
import time
from collections.abc import Callable
from typing import cast

import numpy as np
from numpy.typing import NDArray

from _aegis_game.decorator import requires

from .aegis_config import get_feature_value, has_feature
from .agent import Agent
from .agent_controller import AgentController
from .agent_predictions.prediction_handler import PredictionHandler
from .agent_type import AgentType
from .args_parser import LaunchArgs
from .common import Cell, CellInfo, Direction, Location
from .common.objects import Rubble, Survivor
from .constants import Constants
from .game_pb import GamePb
from .id_gen import IDGenerator
from .logger import LOGGER
from .sandbox.sandbox import Sandbox
from .team import Team
from .team_info import TeamInfo
from .types import GameOverReason, MethodDict
from .world import World


class Game:
    def __init__(
        self,
        code: list[Sandbox | None],
        args: LaunchArgs,
        world: World,
        game_pb: GamePb,
    ) -> None:
        random.seed(world.seed)
        self.code: list[Sandbox | None] = code
        self.args: LaunchArgs = args
        self.running: bool = True
        self.reason: GameOverReason | None = None
        self.world: World = world
        self.round: int = 0
        self.id_gen: IDGenerator = IDGenerator()
        self.team_info: TeamInfo = TeamInfo()
        self.team_info.add_lumens(Team.GOOBS, Constants.INITIAL_TEAM_LUMENS)
        self.team_info.add_lumens(Team.VOIDSEERS, Constants.INITIAL_TEAM_LUMENS)
        self.game_pb: GamePb = game_pb
        # key is location, value is team -> num of agents queuing to remove the layer this round
        self._queued_layers_to_remove: dict[Location, dict[Team, int]] = {}
        self._drone_scans: dict[Location, dict[Team, int]] = {}
        self._pending_drone_scans: dict[Location, dict[Team, int]] = {}
        self._prediction_handler: PredictionHandler | None = (
            PredictionHandler(args) if has_feature("ALLOW_AGENT_PREDICTIONS") else None
        )
        self.agents: dict[int, Agent] = {}
        self.team_agents: dict[Team, str] = {}
        if self.args.agent is not None:
            self.team_agents[Team.GOOBS] = self.args.agent
        if self.args.agent2 is not None:
            self.team_agents[Team.VOIDSEERS] = self.args.agent2
        self._init_spawn()

    def _init_spawn(self) -> None:
        if has_feature("ALLOW_AGENT_TYPES"):
            # if agent types enabled, spawn one commander at a random spawn location for each team (team needs to spawn rest of agents)

            spawns = self.get_spawns()
            spawn_loc = random.choice(spawns)

            self._spawn_agents_at(spawn_loc, 1)

        else:
            # if no agent types, spawn commanders up to agent amount specified

            spawns = self.world.init_spawns

            # spawns that need to be filled
            positive_spawns = {loc: amt for loc, amt in spawns.items() if amt > 0}

            remaining = self.args.amount

            for loc, amt in positive_spawns.items():
                # if remaining <= 0:
                #     # really shouldn't be intended by user for this to happen, almost tempted to raise an error here saying "run world with minimum self.args.amount agents"
                #     return

                # to_spawn = min(
                #     amt, remaining
                # )  # should also not have to take the min imo for same reason

                # if we run out of remaining agents before filling all amt's, raise an error
                if remaining < amt:
                    required_agents = sum(amt for amt in positive_spawns.values())
                    msg = f"Not enough agents to fill world spawns! The world requires at least {required_agents} agents, but only {self.args.amount} were provided. Please use {required_agents} agents on this world."
                    raise ValueError(msg)

                self._spawn_agents_at(loc, amt)
                remaining -= amt

            # prio spawns filled, choose from any spawn

            if len(positive_spawns) > 0 and remaining > 0:
                LOGGER.warning(
                    f"Ran world with {self.args.amount} agents, but world only specifies spawns for {sum(amt for amt in positive_spawns.values())} agents. Ensure this was intended usage."
                )

            all_spawns = self.get_spawns()
            while remaining > 0:
                loc = random.choice(all_spawns)
                self._spawn_agents_at(loc, 1)
                remaining -= 1

    def _spawn_agents_at(self, loc: Location, count: int) -> None:
        if self.args.agent is not None:
            for _ in range(count):
                self.spawn_agent(loc, Team.GOOBS, AgentType.COMMANDER)
        if self.args.agent2 is not None:
            for _ in range(count):
                self.spawn_agent(loc, Team.VOIDSEERS, AgentType.COMMANDER)

    def _run_turn(self, agent: Agent) -> None:
        start = time.perf_counter()
        agent.turn()
        end = time.perf_counter()
        duration = end - start
        if duration >= Constants.MAX_TURN_TIME_LIMIT:
            LOGGER.warning(
                f"{agent.id}'s turn took {duration:.2f}s (over {Constants.MAX_TURN_TIME_LIMIT}s limit)"
            )
            self.kill_agent(agent.id)

    def run_round(self) -> None:
        self.tick_drone_scans()
        self.round += 1
        self.game_pb.start_round(self.round)
        self.team_info.add_lumens(Team.GOOBS, Constants.LUMENS_PER_ROUND)
        self.team_info.add_lumens(Team.VOIDSEERS, Constants.LUMENS_PER_ROUND)
        self.for_each_agent(self._run_turn)
        self.rotate_message_buffers()
        self.activate_pending_drone_scans()
        self.game_pb.send_drone_scan_update(self._drone_scans)
        self.apply_survivor_health_decay()
        self.grim_reaper()
        self.serialize_team_info()
        self.serialize_drone_scans()
        self.game_pb.end_round()
        self.check_game_over()

    def rotate_message_buffers(self) -> None:
        """
        Advance all agents' message buffers to the next round.

        This commits any pending messages and ensures that all agents' buffers
        are synchronized for the upcoming round.
        """
        for agent_id in self.agents:
            agent = self.get_agent(agent_id)
            agent.message_buffer.next_round(self.round + 1)

    def check_game_over(self) -> None:
        if self.round == self.world.rounds and self.reason is None:
            self.reason = GameOverReason.MAX_ROUNDS_REACHED

        saved_survs = self.team_info.get_saved(Team.GOOBS)
        saved_survs += self.team_info.get_saved(Team.VOIDSEERS)
        if saved_survs == self.world.total_survivors and self.reason is None:
            self.reason = GameOverReason.ALL_SURVIVORS_SAVED

        if self.reason is not None:
            self.stop()
            if has_feature("ADVANCED_SCORING_SYSTEM"):
                for team in Team:
                    alive_agents = self.team_info.get_units(team)
                    alive_agent_score = alive_agents * Constants.ALIVE_AGENT_SCORE
                    self.team_info.add_score(team, alive_agent_score)

    def grim_reaper(self) -> None:
        dead_agents: list[Agent] = []

        for agent in self.agents.values():
            died = False
            cell = self.get_cell_at(agent.location)
            if agent.energy_level <= 0:
                LOGGER.info("Agent %s ran out of energy and died.\n", agent.id)
                died = True
            elif cell and cell.is_killer_cell():
                LOGGER.info("Agent %s ran into killer cell and died.\n", agent.id)
                died = True

            if died:
                dead_agents.append(agent)

        for agent in dead_agents:
            self.kill_agent(agent.id)

        self.process_layers_queued_to_be_removed()

    def stop(self) -> None:
        self.running = False
        self.for_each_agent(lambda agent: self.kill_agent(agent.id))

    def end_if_no_units(self, _team: Team) -> None:
        if self.reason is not None:
            return

        if len(self.agents) != 0:
            return

        # units = self.team_info.get_units(team)
        # if units != 0:
        #     return

        self.reason = GameOverReason.ALL_AGENTS_DEAD

    def kill_agent(self, agent_id: int) -> None:
        agent = self.agents[agent_id]
        del self.agents[agent_id]
        self.remove_agent_from_loc(agent_id, agent.location)
        agent.kill()
        self.game_pb.add_dead(agent_id)
        self.team_info.add_units(agent.team, -1)
        self.end_if_no_units(agent.team)

    def spawn_agent(
        self,
        loc: Location,
        team: Team,
        agent_type: AgentType,
        agent_id: int | None = None,
    ) -> None:
        agent_id = self.id_gen.next_id() if agent_id is None else agent_id
        energy = int(self.world.start_energy * agent_type.energy_multiplier)
        agent = Agent(self, agent_id, loc, team, energy, agent_type)
        ac = AgentController(self, agent)
        agent.launch(self.code[team.value], self.methods(ac), debug=self.args.debug)
        self.add_agent(agent, loc)
        self.team_info.add_units(agent.team, 1)
        self.game_pb.add_spawn(agent.id, agent.team, agent.location)

    def add_agent(self, agent: Agent, loc: Location) -> None:
        if agent not in self.agents:
            self.agents[agent.id] = agent

            cell = self.get_cell_at(loc)
            cell.agents.append(agent.id)
            LOGGER.info("Added agent %s", agent.id)

    def get_agent(self, agent_id: int) -> Agent:
        return self.agents[agent_id]

    def for_each_agent(self, fn: Callable[[Agent], None]) -> None:
        for agent in list(self.agents.values()):
            fn(agent)

    def queue_layer_to_remove(self, loc: Location, team: Team) -> None:
        # init the Location, so it will have its layer removed this round
        if loc not in self._queued_layers_to_remove:
            self._queued_layers_to_remove[loc] = {}

        # add this team to list of teams wanting to remove the layer this round
        if team not in self._queued_layers_to_remove[loc]:
            self._queued_layers_to_remove[loc][team] = 1
        else:
            self._queued_layers_to_remove[loc][team] += 1

    def process_layers_queued_to_be_removed(self) -> None:
        # check if each loc should actually have its top layer removed this round
        for loc, teams_data in self._queued_layers_to_remove.items():
            # set true if at least one team meets threshold to remove the layer
            will_remove = False
            agents_needed_to_remove = 1
            top_layer = self.get_cell_info_at(loc).top_layer
            if isinstance(top_layer, Rubble):
                agents_needed_to_remove = top_layer.agents_required
            # see if each team met threshold to acc remove the layer
            for team, num_agents_queued in teams_data.items():
                if num_agents_queued >= agents_needed_to_remove:
                    will_remove = True
                    # award team properly for whatever layer they just removed
                    self.reward_layer_removal(loc, team)

            if will_remove:
                # ensures we only call remove_layer ONCE per loc per round
                self.remove_layer(loc)

        self._queued_layers_to_remove.clear()

    def reward_layer_removal(self, loc: Location, team: Team) -> None:
        top_layer = self.get_cell_info_at(loc).top_layer

        if isinstance(top_layer, Survivor):
            points = 0
            decay_rate = get_feature_value("SURV_HEALTH_DECAY_RATE")
            if decay_rate is not None and decay_rate > 0:
                points = top_layer.health
            else:
                points = (
                    Constants.SURVIVOR_SAVE_ALIVE_SCORE
                    if top_layer.is_alive()
                    else Constants.SURVIVOR_SAVE_DEAD_SCORE
                )

            self.team_info.add_saved(team, 1, is_alive=top_layer.is_alive())
            self.team_info.add_score(team, points)

    def remove_layer(self, loc: Location) -> None:
        cell = self.get_cell_at(loc)
        _ = cell.remove_top_layer()
        self.game_pb.add_removed_layer(loc)

    def mark_surrounding_cells_visited(self, agent: Agent, loc: Location) -> None:
        for direction in Direction:
            if direction == Direction.CENTER:
                continue

            new_loc = loc.add(direction)
            if not self.on_map(new_loc):
                continue

            index = new_loc.x + new_loc.y * self.world.width
            agent.has_visited[index] = True

    def add_agent_to_loc(self, agent_id: int, loc: Location) -> None:
        self.get_cell_at(loc).agents.append(agent_id)
        agent = self.get_agent(agent_id)
        if has_feature("HIDDEN_MOVE_COSTS"):
            self.mark_surrounding_cells_visited(agent, loc)

    def remove_agent_from_loc(self, agent_id: int, loc: Location) -> None:
        self.get_cell_at(loc).agents.remove(agent_id)

    def move_agent(self, agent_id: int, start_loc: Location, end_loc: Location) -> None:
        self.remove_agent_from_loc(agent_id, start_loc)
        self.add_agent_to_loc(agent_id, end_loc)

    def start_drone_scan(self, loc: Location, team: Team) -> None:
        if loc not in self._pending_drone_scans:
            self._pending_drone_scans[loc] = {}
        self._pending_drone_scans[loc][team] = Constants.DRONE_SCAN_DURATION

    def activate_pending_drone_scans(self) -> None:
        """Activate pending drone scans by moving them to active drone scans."""
        for loc, teams in self._pending_drone_scans.items():
            if loc not in self._drone_scans:
                self._drone_scans[loc] = {}
            for team, duration in teams.items():
                self._drone_scans[loc][team] = duration
                LOGGER.info(
                    f"Started drone scan at {loc} for team {team.name} with duration of {duration} rounds"
                )
        self._pending_drone_scans.clear()

    def is_loc_drone_scanned(self, loc: Location, team: Team) -> bool:
        return loc in self._drone_scans and team in self._drone_scans[loc]

    def get_drone_scan_duration(self, loc: Location, team: Team) -> int:
        return self._drone_scans[loc][team]

    def tick_drone_scans(self) -> None:
        for loc, teams in self._drone_scans.items():
            for team, duration in list(teams.items()):
                teams[team] = duration - 1
                if teams[team] <= 0:
                    del self._drone_scans[loc][team]

    def serialize_drone_scans(self) -> None:
        """Add all active drone scans to the protobuf data for this round."""
        for loc, teams in self._drone_scans.items():
            for team, duration in teams.items():
                self.game_pb.add_drone_scan(loc, team, duration)

    def apply_survivor_health_decay(self) -> None:
        """Apply health decay to all survivors based on config setting."""
        decay_rate = get_feature_value("SURV_HEALTH_DECAY_RATE")
        if decay_rate is None or decay_rate <= 0:
            return  # Decay rate of 0 turns off health decay

        for cell in self.world.cells:
            for layer in cell.layers:
                if isinstance(layer, Survivor) and layer.is_alive():
                    layer.health = max(0, layer.health - decay_rate)

                    if layer.health <= 0:
                        layer.set_state(Survivor.State.DEAD)
                        LOGGER.info(f"Survivor {layer.id} died from health decay")

                    # Track health change for client (only once per survivor per round)
                    self.game_pb.add_survivor_health_update(
                        cell.location, layer.id, layer.health, layer.get_state().value
                    )

    def save(self, survivor: Survivor, agent: Agent) -> None:
        if (
            agent.location in self._queued_layers_to_remove
            and agent.team in self._queued_layers_to_remove[agent.location]
        ):
            LOGGER.info(
                f"Skipping saving survivor {survivor.id} at {agent.location} for team {agent.team} because someone else on that team saved this surv already"
            )
            return

        agent.add_energy(-Constants.SAVE_ENERGY_COST)
        self.queue_layer_to_remove(agent.location, agent.team)

        LOGGER.info(
            f"Saving survivor {survivor.id} at {agent.location} for team {agent.team.name} on round {self.round}"
        )
        if (
            has_feature("ALLOW_AGENT_PREDICTIONS")
            and self._prediction_handler is not None
        ):
            LOGGER.info(
                f"Creating pending prediction for team {agent.team} and surv_id {survivor.id}"
            )
            self._prediction_handler.create_pending_prediction(
                agent.team,
                survivor.id,
            )

    def dig(self, agent: Agent) -> None:
        rubble_energy_required = cast(
            "Rubble", self.get_cell_info_at(agent.location).top_layer
        ).energy_required
        if agent.energy_level < rubble_energy_required:
            return

        agent.add_energy(-rubble_energy_required)
        self.queue_layer_to_remove(agent.location, agent.team)

    def predict(self, surv_id: int, label: np.int32, agent: Agent) -> None:
        if (
            not has_feature("ALLOW_AGENT_PREDICTIONS")
            or self._prediction_handler is None
        ):
            return

        is_correct = self._prediction_handler.predict(agent.team, surv_id, label)

        if is_correct is None:
            LOGGER.warning(
                f"Agent {agent.id} attempted invalid prediction for surv_id {surv_id}"
            )
            return
        score = Constants.PRED_CORRECT_SCORE if is_correct else 0
        self.team_info.add_score(agent.team, score)
        self.team_info.add_predicted(agent.team, 1, correct=is_correct)

    def on_map(self, loc: Location) -> bool:
        """
        Check whether a location is within the bounds of the world.

        Args:
            loc: The location to check.

        Returns:
            True if the location is on the map, False otherwise.

        """
        return 0 <= loc.x < self.world.width and 0 <= loc.y < self.world.height

    def get_cell_at(self, loc: Location) -> Cell:
        index = loc.x + loc.y * self.world.width
        return self.world.cells[index]

    def get_cell_info_at(self, location: Location) -> CellInfo:
        cell = self.get_cell_at(location)
        return CellInfo(
            cell.layers, cell.type, cell.location, cell.move_cost, cell.agents
        )

    def serialize_team_info(self) -> None:
        self.game_pb.add_team_info(Team.GOOBS, self.team_info)
        self.game_pb.add_team_info(Team.VOIDSEERS, self.team_info)

    def get_survs(self) -> list[Location]:
        """Return a list of survivor locations."""
        return [
            cell.location for cell in self.world.cells if cell.number_of_survivors() > 0
        ]

    @requires("ALLOW_AGENT_TYPES")
    def get_spawns(self) -> list[Location]:
        """Return a list of spawn locations."""
        return [cell.location for cell in self.world.cells if cell.is_spawn()]

    def get_charging_cells(self) -> list[Location]:
        """Return a list of charging locations."""
        return [cell.location for cell in self.world.cells if cell.is_charging_cell()]

    def get_prediction_info_for_agent(
        self, team: Team
    ) -> list[tuple[int, NDArray[np.uint8], NDArray[np.int32]]]:
        if (
            not has_feature("ALLOW_AGENT_PREDICTIONS")
            or self._prediction_handler is None
        ):
            return []
        return self._prediction_handler.read_pending_predictions(team)

    def methods(self, ac: AgentController) -> MethodDict:
        return {
            "AgentType": AgentType,
            "Direction": Direction,
            "Location": Location,
            "Rubble": Rubble,
            "Survivor": Survivor,
            "get_round_number": ac.get_round_number,
            "get_id": ac.get_id,
            "get_type": ac.get_type,
            "get_team": ac.get_team,
            "get_location": ac.get_location,
            "get_energy_level": ac.get_energy_level,
            "get_lumens": ac.get_lumens,
            "get_cell_info_at": ac.get_cell_info_at,
            "send_message": ac.send_message,
            "read_messages": ac.read_messages,
            "drone_scan": ac.drone_scan,
            "move": ac.move,
            "save": ac.save,
            "dig": ac.dig,
            "recharge": ac.recharge,
            "predict": ac.predict,
            "read_pending_predictions": ac.read_pending_predictions,
            "spawn_agent": ac.spawn_agent,
            "on_map": self.on_map,
            "get_charging_cells": self.get_charging_cells,
            "get_spawns": self.get_spawns,
            "get_survs": self.get_survs,
            "log": ac.log,
        }



# pyright: reportAny = false
# pyright: reportExplicitAny = false
import platform
import shutil
import subprocess
import sys
import zipfile
from pathlib import Path
from typing import Any, ClassVar

import requests


class ClientInstaller:
    """Handles downloading and installing the AEGIS client."""

    OWNER: str = "AEGIS-GAME"
    REPO: str = "aegis"
    CHUNK_SIZE: int = 8192
    CLIENT_DIR: Path = Path("client")

    PLATFORM_ASSETS: ClassVar[dict[str, str]] = {
        "windows": "win-client.zip",
        "darwin": "mac-client.zip",
        "linux": "linux-client.zip",
    }

    EXECUTABLE_PATTERNS: ClassVar[list[str]] = ["*.app", "*.AppImage", "*.exe"]

    def __init__(self) -> None:
        self.platform: str = self._detect_platform()
        self.asset_name: str = self.PLATFORM_ASSETS[self.platform]

    def _detect_platform(self) -> str:
        """Detect the current platform."""
        system = platform.system().lower()

        if system.startswith("win"):
            return "windows"
        if system.startswith("darwin"):
            return "darwin"
        if system.startswith("linux"):
            return "linux"
        sys.exit(f"Unsupported platform: {system}")

    def _get_latest_release(self) -> dict[str, Any]:
        """Fetch the latest client release from GitHub."""
        url = f"https://api.github.com/repos/{self.OWNER}/{self.REPO}/releases/latest"

        try:
            response = requests.get(url, timeout=10)
            response.raise_for_status()
        except requests.RequestException as e:
            sys.exit(f"Failed to fetch releases: {e}")

        release = response.json()

        if not release:
            sys.exit("No client release found")

        return release

    def _find_asset(self, release: dict[str, Any]) -> dict[str, Any]:
        """Find the matching asset in the GitHub release."""
        for asset in release["assets"]:
            if asset["name"] == self.asset_name:
                return asset

        sys.exit(
            f"Asset '{self.asset_name}' not found in release {release['tag_name']}"
        )

    def _download_with_progress(self, url: str, output_path: Path) -> None:
        """Download client zip file with progress indicator."""
        try:
            response = requests.get(url, stream=True, timeout=10)
            response.raise_for_status()
        except requests.RequestException as e:
            sys.exit(f"Download failed: {e}")

        total_size = int(response.headers.get("Content-Length", 0))
        downloaded = 0

        with output_path.open("wb") as f:
            for chunk in response.iter_content(chunk_size=self.CHUNK_SIZE):
                if not chunk:
                    continue

                _ = f.write(chunk)
                downloaded += len(chunk)
                self._show_progress(downloaded, total_size)

        print(f"\nDownload complete: {output_path}")

    def _show_progress(self, downloaded: int, total_size: int) -> None:
        """Display download progress."""
        downloaded_mb = downloaded / (1024 * 1024)

        if total_size:
            total_mb = total_size / (1024 * 1024)
            percent = (downloaded / total_size) * 100
            print(
                f"Downloaded {downloaded_mb:.2f} MB / {total_mb:.2f} MB ({percent:.1f}%)",
                end="\r",
            )
        else:
            print(f"Downloaded {downloaded_mb:.2f} MB", end="\r")

    def _extract_archive(self, zip_path: Path) -> None:
        """Extract the downloaded archive."""
        if self.CLIENT_DIR.exists():
            shutil.rmtree(self.CLIENT_DIR)

        self.CLIENT_DIR.mkdir(parents=True, exist_ok=True)

        try:
            if self.platform == "windows":
                with zipfile.ZipFile(zip_path, "r") as zip_file:
                    zip_file.extractall(self.CLIENT_DIR)
            else:
                _ = subprocess.run(  # noqa: S603
                    ["unzip", "-o", str(zip_path), "-d", str(self.CLIENT_DIR)],  # noqa: S607
                    check=True,
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.DEVNULL,
                )
        except Exception as e:  # noqa: BLE001
            sys.exit(f"Failed to extract archive: {e}")

        print(f"Extracted to: {self.CLIENT_DIR}")

    def _find_executable(self) -> Path | None:
        """Find the main executable in the extracted files."""
        for pattern in self.EXECUTABLE_PATTERNS:
            matches = list(self.CLIENT_DIR.rglob(pattern))
            if matches:
                return matches[0]
        return None

    def _move_executable_to_root(self) -> str | None:
        """Move the main executable to the root of CLIENT_DIR."""
        executable = self._find_executable()

        if not executable:
            print("Warning: No executable found")
            return None

        if executable.parent == self.CLIENT_DIR:
            print(f"Executable already at root: {executable.name}")
            return executable.name

        destination = self.CLIENT_DIR / executable.name

        if destination.exists():
            if destination.is_dir():
                shutil.rmtree(destination)
            else:
                destination.unlink()

        _ = shutil.move(str(executable), str(destination))
        print(f"Moved executable to: {destination}")

        return executable.name

    def _cleanup_directory(self, keep_name: str) -> None:
        """Remove all files except the main executable."""
        for item in self.CLIENT_DIR.iterdir():
            if item.name == keep_name:
                continue

            if item.is_dir():
                shutil.rmtree(item)
            else:
                item.unlink()

        print(f"Cleaned directory, kept only: {keep_name}")

    def install(self) -> None:
        """Download and install the latest client."""
        zip_path = Path(self.asset_name)

        if zip_path.exists():
            print(f"Archive '{zip_path}' already exists, skipping download")
        else:
            print("Fetching latest release...")
            release = self._get_latest_release()
            asset = self._find_asset(release)

            print(f"Downloading {self.asset_name} from {release['tag_name']}...")
            self._download_with_progress(asset["browser_download_url"], zip_path)

        print("Extracting client files...")
        self._extract_archive(zip_path)

        executable_name = self._move_executable_to_root()

        if executable_name:
            self._cleanup_directory(executable_name)
            print(f"Installation complete! Client available in: {self.CLIENT_DIR}")
        else:
            print("Installation completed with warnings")


def main() -> None:
    """Entry point for the client installer."""
    installer = ClientInstaller()
    installer.install()



from .common import Cell, Location
from .constants import Constants


class World:
    def __init__(  # noqa: PLR0913
        self,
        width: int,
        height: int,
        seed: int,
        start_energy: int,
        cells: list[Cell],
        init_spawns: dict[Location, int],
    ) -> None:
        self.width: int = width
        self.height: int = height
        self.rounds: int = Constants.DEFAULT_MAX_ROUNDS
        self.seed: int = seed
        self.start_energy: int = start_energy
        self.cells: list[Cell] = cells
        self.total_survivors: int = sum(cell.number_of_survivors() for cell in cells)
        self.init_spawns: dict[Location, int] = init_spawns

        self._validate_map()

    def _validate_map(self) -> None:
        min_size = Constants.WORLD_MIN
        max_size = Constants.WORLD_MAX

        if self.width < min_size:
            error = f"World width must be at least {min_size}"
            raise ValueError(error)

        if self.width > max_size:
            error = f"World width must not exceed {max_size}"
            raise ValueError(error)

        if self.height < min_size:
            error = f"World height must be at least {min_size}"
            raise ValueError(error)

        if self.height > max_size:
            error = f"World height must not exceed {max_size}"
            raise ValueError(error)



"""Public Aegis export stuff."""

from _aegis_game.agent_type import AgentType
from _aegis_game.cli import main
from _aegis_game.common import CellInfo, Direction, Location
from _aegis_game.common.objects import Rubble, Survivor
from _aegis_game.message import Message
from _aegis_game.team import Team

__all__ = [
    "AgentType",
    "CellInfo",
    "Direction",
    "Location",
    "Message",
    "Rubble",
    "Survivor",
    "Team",
    "main",
]



# noqa: INP001

"""Generate index.aegis.ts and index.ts for TS schema."""

from pathlib import Path

TS_DIR = Path(__file__).parent.parent / "ts"
INDEX_AEGIS = TS_DIR / "index.aegis.ts"
INDEX_MAIN = TS_DIR / "index.ts"


def generate_index_aegis_ts() -> None:
    """Make index.aegis.ts with exports."""
    exports: list[str] = []

    for ts_file in TS_DIR.glob("*.ts"):
        if ts_file.name in ("index.ts", "index.aegis.ts"):
            continue
        module = ts_file.stem
        exports.append(f"export * from './{module}';")

    index_content = "\n".join(exports) + "\n"
    _ = INDEX_AEGIS.write_text(index_content)
    print(f"Generated {INDEX_AEGIS} with {len(exports)} exports.")


def generate_index_ts() -> None:
    """Make index.ts with schema namespace import."""
    index_content = "export * as schema from './index.aegis'\n"
    _ = INDEX_MAIN.write_text(index_content)
    print(f"Generated {INDEX_MAIN} with schema namespace import.")


if __name__ == "__main__":
    generate_index_aegis_ts()
    generate_index_ts()



import shutil
from importlib import resources
from pathlib import Path


def _find_project_root() -> Path:
    """Find the project root directory by looking for .venv."""
    current = Path.cwd()
    while current != current.parent:
        if (current / ".venv").exists():
            return current
        current = current.parent
    msg = "Could not find project root (.venv)"
    raise FileNotFoundError(msg)


def _get_templates_dir(kind: str) -> Path:
    """
    Get the templates directory path for the given template kind.

    This works both for development (source code) and PyPI installation.
    """
    try:
        # First try to find templates in the installed package
        with resources.path("_aegis_game.templates", kind) as template_path:
            return template_path
    except (ImportError, FileNotFoundError):
        # Fallback to development path (source code)
        project_root = _find_project_root()
        dev_templates_dir = project_root / "src" / "_aegis_game" / "templates" / kind
        if dev_templates_dir.exists():
            return dev_templates_dir

        # If neither exists, raise a clear error
        msg = f"Template '{kind}' not found. Please ensure templates are populated."
        raise FileNotFoundError(msg) from None


def _copy_directory_excluding_cache(src: Path, dest: Path) -> None:
    """Copy directory excluding __pycache__ and other cache files."""

    def ignore_patterns(_: str, names: list[str]) -> list[str]:
        """Ignore __pycache__, .pyc, and other cache files."""
        return [
            name
            for name in names
            if name in {"__pycache__", ".pyc", ".pyo", ".pyd"}
            or name.endswith((".pyc", ".pyo", ".pyd"))
        ]

    _ = shutil.copytree(src, dest, ignore=ignore_patterns)


def _copy_template_to_workspace(
    template_dir: Path, workspace_dir: Path
) -> tuple[bool, bool, bool, bool]:
    """Copy template files to the workspace directory."""
    created_world = False
    created_agent = False
    created_config = False
    created_predictions = False

    # Copy worlds
    world_src = template_dir / "worlds"
    world_dest = workspace_dir / "worlds"
    if world_src.exists():
        if not world_dest.exists():
            _copy_directory_excluding_cache(world_src, world_dest)
            created_world = True
        else:
            print("  - worlds/: exists, skipped")

    # Copy agents
    agent_src = template_dir / "agents"
    agent_dest = workspace_dir / "agents"
    if agent_src.exists():
        if not agent_dest.exists():
            _copy_directory_excluding_cache(agent_src, agent_dest)
            created_agent = True
        else:
            print("  - agents/: exists, skipped")

    # Copy config
    config_src = template_dir / "config"
    config_dest = workspace_dir / "config"
    if config_src.exists():
        if not config_dest.exists():
            _copy_directory_excluding_cache(config_src, config_dest)
            created_config = True
        else:
            print("  - config/: exists, skipped")

    # Copy prediction data
    predictions_src = template_dir / "prediction_data"
    predictions_dest = workspace_dir / "prediction_data"
    if predictions_src.exists():
        if not predictions_dest.exists():
            _copy_directory_excluding_cache(predictions_src, predictions_dest)
            created_predictions = True
        else:
            print("  - prediction_data/: exists, skipped")

    return created_world, created_agent, created_config, created_predictions


def _print_summary(  # noqa: PLR0913
    *,
    created_world: bool,
    created_agent: bool,
    created_config: bool,
    created_predictions: bool,
    include_predictions: bool,
    agent_folder_name: str,
) -> None:
    print("[aegis] Init scaffold:")
    print(
        f"\t- worlds/ExampleWorld.world: {'copied' if created_world else 'exists, skipped'}"
    )
    print(
        f"\t- agents/{agent_folder_name}: {'copied' if created_agent else 'exists, skipped'}"
    )
    print(
        f"\t- config/config.yaml: {'copied' if created_config else 'exists, skipped'}"
    )
    if include_predictions:
        print(
            f"\t- prediction_data: {'copied' if created_predictions else 'exists, skipped'}"
        )


def init_scaffold(kind: str = "path") -> None:
    """
    Copy predefined scaffold files into the current working directory.

    kind: one of "path" (default), "mas", or "comp".
    """
    cwd = Path.cwd()

    # Find the templates directory (works for both dev and PyPI installation)
    try:
        templates_dir = _get_templates_dir(kind)
    except FileNotFoundError as e:
        msg = f"[aegis] init failed: {e}"
        raise FileNotFoundError(msg) from e

    print(f"[aegis] Initializing {kind} template from {templates_dir}")

    # Copy the entire template structure to the current working directory
    created_world, created_agent, created_config, created_predictions = (
        _copy_template_to_workspace(templates_dir, cwd)
    )

    # Determine agent folder name and whether to include predictions
    agent_folder_name = (
        "agent_path"
        if kind == "path"
        else "agent_mas"
        if kind == "mas"
        else "agent_comp"
    )
    include_predictions = kind in {"mas", "comp"}

    _print_summary(
        created_world=created_world,
        created_agent=created_agent,
        created_config=created_config,
        created_predictions=created_predictions,
        include_predictions=include_predictions,
        agent_folder_name=agent_folder_name,
    )



# pyright: reportMissingTypeStubs = false
# pyright: reportUnknownMemberType = false
from ast import (
    AST,
    AnnAssign,
    ClassDef,
    Constant,
    Expr,
    Name,
    ParamSpec,
    TypeAlias,
    TypeVar,
    TypeVarTuple,
    stmt,
)
from typing import override

from RestrictedPython import RestrictingNodeTransformer


class NodeTransformer(RestrictingNodeTransformer):
    """Allow type annoation in RestrictedPython."""

    def doc_str(self, node: stmt) -> str | None:
        if (
            isinstance(node, Expr)
            and isinstance(node.value, Constant)
            and isinstance(node.value.value, str)
        ):
            return node.value.value
        return None

    def get_descriptions(self, body: list[stmt]) -> dict[str, str | None]:
        doc_strings: dict[str, str | None] = {}
        current_name = None
        for node in body:
            if isinstance(node, AnnAssign) and isinstance(node.target, Name):
                current_name = node.target.id
                continue
            if current_name and self.doc_str(node):
                doc_strings[current_name] = self.doc_str(node)
            current_name = None
        return doc_strings

    @override
    def visit_AnnAssign(self, node: AnnAssign) -> AST:
        return self.node_contents_visit(node)

    @override
    def visit_TypeAlias(self, node: TypeAlias) -> AST:
        return self.node_contents_visit(node)

    @override
    def visit_TypeVar(self, node: TypeVar) -> AST:
        return self.node_contents_visit(node)

    @override
    def visit_TypeVarTuple(self, node: TypeVarTuple) -> AST:
        return self.node_contents_visit(node)

    @override
    def visit_ParamSpec(self, node: ParamSpec) -> AST:
        return self.node_contents_visit(node)

    @override
    def visit_ClassDef(self, node: ClassDef) -> stmt:
        # find attribute docs in this class definition
        doc_strings = self.get_descriptions(node.body)
        self.used_names[node.name + ":doc_strings"] = doc_strings
        return super().visit_ClassDef(node)



from .team import Team


class TeamInfo:
    def __init__(self) -> None:
        self._saved_alive: list[int] = [0] * 2
        self._saved_dead: list[int] = [0] * 2
        self._saved: list[int] = [0] * 2
        self._predicted_right: list[int] = [0] * 2
        self._predicted_wrong: list[int] = [0] * 2
        self._predicted: list[int] = [0] * 2
        self._score: list[int] = [0] * 2
        self._units: list[int] = [0] * 2
        self._lumens: list[int] = [0] * 2

    def get_saved(self, team: Team) -> int:
        return self._saved[team.value]

    def get_saved_alive(self, team: Team) -> int:
        return self._saved_alive[team.value]

    def get_saved_dead(self, team: Team) -> int:
        return self._saved_dead[team.value]

    def get_predicted(self, team: Team) -> int:
        return self._predicted[team.value]

    def get_predicted_right(self, team: Team) -> int:
        return self._predicted_right[team.value]

    def get_predicted_wrong(self, team: Team) -> int:
        return self._predicted_wrong[team.value]

    def get_score(self, team: Team) -> int:
        return self._score[team.value]

    def get_units(self, team: Team) -> int:
        return self._units[team.value]

    def get_lumens(self, team: Team) -> int:
        return self._lumens[team.value]

    def _add(self, array: list[int], team: Team, amount: int = 1) -> None:
        array[team.value] += amount

    def add_saved(self, team: Team, amount: int = 1, *, is_alive: bool) -> None:
        self._add(self._saved, team, amount)
        if is_alive:
            self._add(self._saved_alive, team, amount)
        else:
            self._add(self._saved_dead, team, amount)

    def add_predicted(self, team: Team, amount: int = 1, *, correct: bool) -> None:
        self._add(self._predicted, team, amount)
        if correct:
            self._add(self._predicted_right, team, amount)
        else:
            self._add(self._predicted_wrong, team, amount)

    def add_score(self, team: Team, amount: int) -> None:
        self._add(self._score, team, amount)

    def add_units(self, team: Team, amount: int) -> None:
        self._add(self._units, team, amount)

    def add_lumens(self, team: Team, amount: int) -> None:
        self._add(self._lumens, team, amount)



from . import game_pb2 as _game_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Event(_message.Message):
    __slots__ = ("games_header", "game_header", "round", "game_footer", "games_footer", "drone_scan_update")
    GAMES_HEADER_FIELD_NUMBER: _ClassVar[int]
    GAME_HEADER_FIELD_NUMBER: _ClassVar[int]
    ROUND_FIELD_NUMBER: _ClassVar[int]
    GAME_FOOTER_FIELD_NUMBER: _ClassVar[int]
    GAMES_FOOTER_FIELD_NUMBER: _ClassVar[int]
    DRONE_SCAN_UPDATE_FIELD_NUMBER: _ClassVar[int]
    games_header: _game_pb2.GamesHeader
    game_header: _game_pb2.GameHeader
    round: _game_pb2.Round
    game_footer: _game_pb2.GameFooter
    games_footer: _game_pb2.GamesFooter
    drone_scan_update: _game_pb2.DroneScanUpdate
    def __init__(self, games_header: _Optional[_Union[_game_pb2.GamesHeader, _Mapping]] = ..., game_header: _Optional[_Union[_game_pb2.GameHeader, _Mapping]] = ..., round: _Optional[_Union[_game_pb2.Round, _Mapping]] = ..., game_footer: _Optional[_Union[_game_pb2.GameFooter, _Mapping]] = ..., games_footer: _Optional[_Union[_game_pb2.GamesFooter, _Mapping]] = ..., drone_scan_update: _Optional[_Union[_game_pb2.DroneScanUpdate, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: game.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'game.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import location_pb2 as location__pb2
from . import spawn_pb2 as spawn__pb2
from . import team_pb2 as team__pb2
from . import turn_pb2 as turn__pb2
from . import world_pb2 as world__pb2
from . import world_object_pb2 as world__object__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\ngame.proto\x12\x05\x61\x65gis\x1a\x0elocation.proto\x1a\x0bspawn.proto\x1a\nteam.proto\x1a\nturn.proto\x1a\x0bworld.proto\x1a\x12world_object.proto\"[\n\tDroneScan\x12!\n\x08location\x18\x01 \x01(\x0b\x32\x0f.aegis.Location\x12\x19\n\x04team\x18\x02 \x01(\x0e\x32\x0b.aegis.Team\x12\x10\n\x08\x64uration\x18\x03 \x01(\x05\"8\n\x0f\x44roneScanUpdate\x12%\n\x0b\x64rone_scans\x18\x01 \x03(\x0b\x32\x10.aegis.DroneScan\"\x8b\x01\n\x14SurvivorHealthUpdate\x12!\n\x08location\x18\x01 \x01(\x0b\x32\x0f.aegis.Location\x12\x13\n\x0bsurvivor_id\x18\x02 \x01(\x05\x12\x12\n\nnew_health\x18\x03 \x01(\x05\x12\'\n\tnew_state\x18\x04 \x01(\x0e\x32\x14.aegis.SurvivorState\"\r\n\x0bGamesHeader\"W\n\nGameHeader\x12\x1b\n\x05world\x18\x01 \x01(\x0b\x32\x0c.aegis.World\x12\x0e\n\x06rounds\x18\x02 \x01(\x05\x12\x1c\n\x06spawns\x18\x03 \x03(\x0b\x32\x0c.aegis.Spawn\"\xf6\x01\n\x05Round\x12\r\n\x05round\x18\x01 \x01(\x05\x12\'\n\x0elayers_removed\x18\x02 \x03(\x0b\x32\x0f.aegis.Location\x12\x10\n\x08\x64\x65\x61\x64_ids\x18\x03 \x03(\x05\x12\x1a\n\x05turns\x18\x04 \x03(\x0b\x32\x0b.aegis.Turn\x12\"\n\tteam_info\x18\x05 \x03(\x0b\x32\x0f.aegis.TeamInfo\x12%\n\x0b\x64rone_scans\x18\x06 \x03(\x0b\x32\x10.aegis.DroneScan\x12<\n\x17survivor_health_updates\x18\x07 \x03(\x0b\x32\x1b.aegis.SurvivorHealthUpdate\"\x0c\n\nGameFooter\"\r\n\x0bGamesFooterb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'game_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_DRONESCAN']._serialized_start=107
  _globals['_DRONESCAN']._serialized_end=198
  _globals['_DRONESCANUPDATE']._serialized_start=200
  _globals['_DRONESCANUPDATE']._serialized_end=256
  _globals['_SURVIVORHEALTHUPDATE']._serialized_start=259
  _globals['_SURVIVORHEALTHUPDATE']._serialized_end=398
  _globals['_GAMESHEADER']._serialized_start=400
  _globals['_GAMESHEADER']._serialized_end=413
  _globals['_GAMEHEADER']._serialized_start=415
  _globals['_GAMEHEADER']._serialized_end=502
  _globals['_ROUND']._serialized_start=505
  _globals['_ROUND']._serialized_end=751
  _globals['_GAMEFOOTER']._serialized_start=753
  _globals['_GAMEFOOTER']._serialized_end=765
  _globals['_GAMESFOOTER']._serialized_start=767
  _globals['_GAMESFOOTER']._serialized_end=780
# @@protoc_insertion_point(module_scope)



from typing import Literal, TypedDict


class WorldSize(TypedDict):
    width: int
    height: int


class WorldInfo(TypedDict):
    size: WorldSize
    seed: int
    start_energy: int


class Loc(TypedDict):
    x: int
    y: int


Attributes = Literal[
    "health",
    "energy_required",
    "agents_required",
]


class Layer(TypedDict):
    type: str
    attributes: dict[Attributes, int]


class CellInfoRaw(TypedDict):
    loc: Loc
    moveCost: int
    type: str | None
    layers: list[Layer]


class WorldRaw(TypedDict):
    world_info: WorldInfo
    cells: list[CellInfoRaw]



from functools import lru_cache
from pathlib import Path
from typing import cast

import yaml

from .types import AegisConfig, FeatureKey

CONFIG_PATH = Path.cwd() / "config" / "config.yaml"


@lru_cache
def load_config() -> AegisConfig:
    """Load and cache the main configuration file."""
    if not CONFIG_PATH.exists():
        error = f"Main config file not found: {CONFIG_PATH}"
        raise FileNotFoundError(error)

    with CONFIG_PATH.open() as f:
        return cast("AegisConfig", yaml.safe_load(f))


def has_feature(feature: FeatureKey) -> bool:
    """Check if a feature is enabled in the config."""
    config = load_config()

    return config.get("features", {}).get(feature, False) or config.get(
        "competition_specific", {}
    ).get(feature, False)


def get_feature_value(feature: FeatureKey) -> bool | int | None:
    """Get a feature value from the config."""
    config = load_config()

    return config.get("features", {}).get(feature) or config.get(
        "competition_specific", {}
    ).get(feature)



"""
Custom script to copy the trash google software (release-please).

DO NOT RUN THIS MANUALLY!!!!!! The workflow will run this when PRs are merged.
"""
# ruff: noqa: S603, S607, PLW1510

import datetime
import json
import subprocess
from pathlib import Path
from typing import cast

import toml
from git import Commit, GitCommandError, Repo
from semver import VersionInfo

PACKAGES = {
    "client": {
        "file": "client/package.json",
        "tag_prefix": "client-v",
        "type": "npm",
        "path": "client",
    },
    "aegis": {
        "file": "pyproject.toml",
        "tag_prefix": "aegis-v",
        "type": "pypi",
        "path": ".",
    },
}


def get_last_tag(repo: Repo, prefix: str) -> str | None:
    """Get the most recent tag."""
    tags = [t.name for t in repo.tags if t.name.startswith(prefix)]
    if not tags:
        return None
    tags.sort(key=lambda x: VersionInfo.parse(x[len(prefix) :]))
    return tags[-1]


def collect_commits_by_package(
    repo: Repo, last_tag: str | None, path_filter: str
) -> list[Commit]:
    """Get the commits from the last tag to HEAD."""
    if last_tag:
        commits = list(repo.iter_commits(f"{last_tag}..HEAD", paths=path_filter))
    else:
        commits = list(repo.iter_commits("HEAD", paths=path_filter))
    return [c for c in commits if "release" not in str(c.message.lower())]


def has_breaking_change(msg: str) -> bool:
    """Return True if commit has BREAKING CHANGE or '!' before the colon."""
    if "BREAKING CHANGE" in msg:
        return True
    header = msg.split(":", 1)[0]
    return "!" in header


def detect_bump(messages: list[str]) -> str | None:
    """Detect bump version."""
    bump = None
    for msg in messages:
        if has_breaking_change(msg):
            return "major"
        if msg.startswith("feat"):
            bump = "minor"
        elif msg.startswith("fix") and bump != "minor":
            bump = "patch"
    return bump


def bump_version(current: str, bump: str) -> str:
    """Bump sem version."""
    v = VersionInfo.parse(current)
    if bump == "major":
        return str(v.bump_major())
    if bump == "minor":
        return str(v.bump_minor())
    return str(v.bump_patch())


def update_version(pkg: str, new_version: str) -> None:
    """Update version in package specific file."""
    path = Path(PACKAGES[pkg]["file"])
    if pkg == "client":
        with path.open() as f:
            data = json.load(f)  # pyright: ignore[reportAny]
        data["version"] = new_version
        with path.open("w") as f:
            json.dump(data, f, indent=2)
            _ = f.write("\n")
    else:
        data = toml.load(path)
        data["project"]["version"] = new_version
        with path.open("w") as f:
            _ = toml.dump(data, f)


def build_package_section(pkg: str, messages: list[str], version: str) -> str:
    """Build a changelog section for a single package."""
    features: list[str] = []
    fixes: list[str] = []
    breaking: list[str] = []
    others: list[str] = []

    for msg in messages:
        header_line = msg.splitlines()[0].strip()
        header = header_line.split(":", 1)[0]

        if "BREAKING CHANGE" in header_line or "!" in header:
            breaking.append(f"- {header_line}")
        elif header_line.startswith("feat"):
            features.append(f"- {header_line}")
        elif header_line.startswith("fix"):
            fixes.append(f"- {header_line}")
        else:
            others.append(f"- {header_line}")

    sections: list[str] = []
    if breaking:
        sections.append("### 💥 Breaking Changes\n" + "\n".join(breaking))
    if features:
        sections.append("### ✨ Features\n" + "\n".join(features))
    if fixes:
        sections.append("### 🐛 Fixes\n" + "\n".join(fixes))
    if others:
        sections.append("### 📝 Other\n" + "\n".join(others))

    section_text = "\n\n".join(sections) if sections else "No notable changes."

    # Wrap in collapsible
    return f"<details>\n  <summary>{pkg} v{version}</summary>\n\n{section_text}\n</details>"


def build_changelog(
    all_commit_messages: dict[str, list[str]], new_versions: dict[str, str]
) -> dict[str, str]:
    """
    Build changelogs for all packages and update their versions.

    Returns a dict mapping each package name to its changelog string.
    """
    changelogs: dict[str, str] = {}

    for pkg, messages in all_commit_messages.items():
        if not messages or pkg not in new_versions:
            continue

        update_version(pkg, new_versions[pkg])

        changelogs[pkg] = build_package_section(pkg, messages, new_versions[pkg])

    return changelogs


def create_or_update_pr(
    new_versions: dict[str, str], changelogs: dict[str, str]
) -> None:
    """Create or update a single combined release PR with per-package changelogs."""
    branch = "release-branch"
    title = f"chore(release): release v{new_versions['aegis']}"

    repo = Repo(".")
    try:
        repo.git.checkout(branch)  # pyright: ignore[reportAny]
        print(f"[*] Pulled existing branch '{branch}' from origin")
        repo.git.pull("origin", branch)  # pyright: ignore[reportAny]
    except GitCommandError:
        print(f"[!] Branch '{branch}' does not exist. Creating new branch.")
        repo.git.checkout("-b", branch)  # pyright: ignore[reportAny]

    today = datetime.datetime.now(tz=datetime.UTC).date()

    for pkg, changelog in changelogs.items():
        if pkg not in new_versions:
            continue

        header = f"## v{new_versions[pkg]} ({today})\n\n"
        if pkg == "client":
            changelog_path = Path("client/CHANGELOG.md")
        else:  # aegis
            changelog_path = Path("CHANGELOG.md")

        old_content = changelog_path.read_text()
        if old_content.startswith("# Changelog"):
            lines = old_content.splitlines(keepends=True)
            first_line = lines[0]
            rest = "".join(lines[1:])
            new_content = first_line + "\n" + header + changelog + "\n\n" + rest
        else:
            new_content = header + changelog + "\n\n" + old_content

        _ = changelog_path.write_text(new_content)
        repo.git.add(str(changelog_path))  # pyright: ignore[reportAny]

    staged_files = repo.git.diff("--cached", "--name-only")  # pyright: ignore[reportAny]
    print(f"[*] Files staged for commit:\n{staged_files}")

    repo.git.config("user.name", "github-actions[bot]")  # pyright: ignore[reportAny]
    repo.git.config("user.email", "github-actions[bot]@users.noreply.github.com")  # pyright: ignore[reportAny]

    try:
        repo.git.commit("-m", title)  # pyright: ignore[reportAny]
    except GitCommandError as e:
        print(f"[*] No changes to commit: {e}")

    repo.git.push("origin", branch)  # pyright: ignore[reportAny]

    pr_list = subprocess.run(
        ["gh", "pr", "list", "--head", branch, "--json", "number"],
        capture_output=True,
        text=True,
    )

    print(f"[*] Existing PRs: {pr_list.stdout}")

    if "[]" in pr_list.stdout:
        body = "\n\n".join(
            changelog for pkg, changelog in changelogs.items() if pkg in new_versions
        )
        _ = subprocess.run(
            [
                "gh",
                "pr",
                "create",
                "--title",
                title,
                "--body",
                body,
                "--base",
                "main",
                "--head",
                branch,
            ],
            check=True,
        )


def main() -> None:
    """Entry point for release script."""
    repo = Repo(".")
    all_commit_messages: dict[str, list[str]] = {}
    bumps: dict[str, str] = {}

    for pkg in PACKAGES:
        print(f"[*] Checking {pkg}...")
        prefix = PACKAGES[pkg]["tag_prefix"]
        last_tag = get_last_tag(repo, prefix)

        commits = collect_commits_by_package(repo, last_tag, PACKAGES[pkg]["path"])

        if pkg == "aegis":
            commits = [
                c
                for c in commits
                if not any(
                    str(f).startswith(PACKAGES["client"]["path"]) for f in c.stats.files
                )
            ]

        messages = [str(c.message.strip()) for c in commits]
        all_commit_messages[pkg] = messages
        bump = detect_bump(messages)
        if bump:
            bumps[pkg] = bump

    if not bumps:
        print("[*] No release needed for any package")
        return

    new_versions: dict[str, str] = {}
    for pkg in PACKAGES:
        if pkg not in bumps:
            continue
        path = Path(PACKAGES[pkg]["file"])
        if pkg == "client":
            with path.open() as f:
                current = cast("str", json.load(f)["version"])
        else:
            data = toml.load(path)
            current = cast("str", data["project"]["version"])

        new_versions[pkg] = bump_version(current, bumps[pkg])

    # TODO: DELETE AFTER FIRST RELEASE
    new_versions = {"client": "2.6.0", "aegis": "2.6.0"}
    changelog_body = build_changelog(all_commit_messages, new_versions)
    create_or_update_pr(new_versions, changelog_body)
    print("[*] Release PR updated")


if __name__ == "__main__":
    main()



from . import location_pb2 as _location_pb2
from . import spawn_pb2 as _spawn_pb2
from . import team_pb2 as _team_pb2
from . import turn_pb2 as _turn_pb2
from . import world_pb2 as _world_pb2
from . import world_object_pb2 as _world_object_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DroneScan(_message.Message):
    __slots__ = ("location", "team", "duration")
    LOCATION_FIELD_NUMBER: _ClassVar[int]
    TEAM_FIELD_NUMBER: _ClassVar[int]
    DURATION_FIELD_NUMBER: _ClassVar[int]
    location: _location_pb2.Location
    team: _team_pb2.Team
    duration: int
    def __init__(self, location: _Optional[_Union[_location_pb2.Location, _Mapping]] = ..., team: _Optional[_Union[_team_pb2.Team, str]] = ..., duration: _Optional[int] = ...) -> None: ...

class DroneScanUpdate(_message.Message):
    __slots__ = ("drone_scans",)
    DRONE_SCANS_FIELD_NUMBER: _ClassVar[int]
    drone_scans: _containers.RepeatedCompositeFieldContainer[DroneScan]
    def __init__(self, drone_scans: _Optional[_Iterable[_Union[DroneScan, _Mapping]]] = ...) -> None: ...

class SurvivorHealthUpdate(_message.Message):
    __slots__ = ("location", "survivor_id", "new_health", "new_state")
    LOCATION_FIELD_NUMBER: _ClassVar[int]
    SURVIVOR_ID_FIELD_NUMBER: _ClassVar[int]
    NEW_HEALTH_FIELD_NUMBER: _ClassVar[int]
    NEW_STATE_FIELD_NUMBER: _ClassVar[int]
    location: _location_pb2.Location
    survivor_id: int
    new_health: int
    new_state: _world_object_pb2.SurvivorState
    def __init__(self, location: _Optional[_Union[_location_pb2.Location, _Mapping]] = ..., survivor_id: _Optional[int] = ..., new_health: _Optional[int] = ..., new_state: _Optional[_Union[_world_object_pb2.SurvivorState, str]] = ...) -> None: ...

class GamesHeader(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class GameHeader(_message.Message):
    __slots__ = ("world", "rounds", "spawns")
    WORLD_FIELD_NUMBER: _ClassVar[int]
    ROUNDS_FIELD_NUMBER: _ClassVar[int]
    SPAWNS_FIELD_NUMBER: _ClassVar[int]
    world: _world_pb2.World
    rounds: int
    spawns: _containers.RepeatedCompositeFieldContainer[_spawn_pb2.Spawn]
    def __init__(self, world: _Optional[_Union[_world_pb2.World, _Mapping]] = ..., rounds: _Optional[int] = ..., spawns: _Optional[_Iterable[_Union[_spawn_pb2.Spawn, _Mapping]]] = ...) -> None: ...

class Round(_message.Message):
    __slots__ = ("round", "layers_removed", "dead_ids", "turns", "team_info", "drone_scans", "survivor_health_updates")
    ROUND_FIELD_NUMBER: _ClassVar[int]
    LAYERS_REMOVED_FIELD_NUMBER: _ClassVar[int]
    DEAD_IDS_FIELD_NUMBER: _ClassVar[int]
    TURNS_FIELD_NUMBER: _ClassVar[int]
    TEAM_INFO_FIELD_NUMBER: _ClassVar[int]
    DRONE_SCANS_FIELD_NUMBER: _ClassVar[int]
    SURVIVOR_HEALTH_UPDATES_FIELD_NUMBER: _ClassVar[int]
    round: int
    layers_removed: _containers.RepeatedCompositeFieldContainer[_location_pb2.Location]
    dead_ids: _containers.RepeatedScalarFieldContainer[int]
    turns: _containers.RepeatedCompositeFieldContainer[_turn_pb2.Turn]
    team_info: _containers.RepeatedCompositeFieldContainer[_team_pb2.TeamInfo]
    drone_scans: _containers.RepeatedCompositeFieldContainer[DroneScan]
    survivor_health_updates: _containers.RepeatedCompositeFieldContainer[SurvivorHealthUpdate]
    def __init__(self, round: _Optional[int] = ..., layers_removed: _Optional[_Iterable[_Union[_location_pb2.Location, _Mapping]]] = ..., dead_ids: _Optional[_Iterable[int]] = ..., turns: _Optional[_Iterable[_Union[_turn_pb2.Turn, _Mapping]]] = ..., team_info: _Optional[_Iterable[_Union[_team_pb2.TeamInfo, _Mapping]]] = ..., drone_scans: _Optional[_Iterable[_Union[DroneScan, _Mapping]]] = ..., survivor_health_updates: _Optional[_Iterable[_Union[SurvivorHealthUpdate, _Mapping]]] = ...) -> None: ...

class GameFooter(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class GamesFooter(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



"""Tests for the Location class."""

from __future__ import annotations

import pytest

from _aegis_game.common.direction import Direction
from _aegis_game.common.location import Location


class TestLocationInit:
    """Tests for Location object initialization."""

    def test_init_sets_coordinate(self) -> None:
        """Test that the constructor correctly assigns x and y."""
        expected_x = 3
        expected_y = 4
        loc = Location(3, 4)
        assert loc.x == expected_x
        assert loc.y == expected_y


class TestAdd:
    """Tests for the `add` method, which moves a Location in a given Direction."""

    @pytest.mark.parametrize(
        ("start", "direction", "expected"),
        [
            (Location(0, 0), Direction.NORTH, Location(0, 1)),
            (Location(5, 5), Direction.SOUTHWEST, Location(4, 4)),
            (Location(2, 7), Direction.NORTHEAST, Location(3, 8)),
            (Location(-2, 3), Direction.CENTER, Location(-2, 3)),
        ],
    )
    def test_add_moves_in_correct_direction(
        self, start: Location, direction: Direction, expected: Location
    ) -> None:
        """Test that `add` returns a new Location offset by the given direction's dx and dy."""
        res = start.add(direction)
        assert res == expected
        assert res is not start


class TestDirectionTo:
    """Tests for the `direction_to` method, which determines the Direction to another Location."""

    @pytest.mark.parametrize(
        ("start", "end", "expected"),
        [
            (Location(0, 0), Location(1, 1), Direction.NORTHEAST),
            (Location(0, 0), Location(1, -1), Direction.SOUTHEAST),
            (Location(0, 0), Location(-1, 1), Direction.NORTHWEST),
            (Location(0, 0), Location(-1, -1), Direction.SOUTHWEST),
            (Location(0, 0), Location(1, 0), Direction.EAST),
            (Location(0, 0), Location(-1, 0), Direction.WEST),
            (Location(0, 0), Location(0, 1), Direction.NORTH),
            (Location(0, 0), Location(0, -1), Direction.SOUTH),
            (Location(2, 2), Location(2, 2), Direction.CENTER),
        ],
    )
    def test_direction_to(
        self, start: Location, end: Location, expected: Direction
    ) -> None:
        """Test that `direction_to` correctly returns the Direction from one Location to another."""
        assert start.direction_to(end) == expected


class TestDistanceTo:
    """Tests for the `distance_to` method, which computes squared Euclidean distance."""

    def test_distance_squared(self) -> None:
        """Test that `distance_to` returns squared Euclidean distance."""
        expected = 25
        assert Location(0, 0).distance_to(Location(3, 4)) == expected
        expected = 0
        assert Location(1, 1).distance_to(Location(1, 1)) == expected
        expected = 52
        assert Location(-1, -1).distance_to(Location(5, 3)) == expected


class TestIsAdjacentTo:
    """Tests for the `is_adjacent_to` method, which checks neighboring positions."""

    @pytest.mark.parametrize(
        ("loc1", "loc2", "expected"),
        [
            (Location(0, 0), Location(0, 1), True),
            (Location(0, 0), Location(1, 0), True),
            (Location(0, 0), Location(1, 1), True),
            (Location(0, 0), Location(2, 0), False),
            (Location(0, 0), Location(0, 2), False),
        ],
    )
    def test_is_adjacent_to(
        self,
        loc1: Location,
        loc2: Location,
        expected: bool,  # noqa: FBT001
    ) -> None:
        """Test that `is_adjacent_to` correctly detects adjacency."""
        assert loc1.is_adjacent_to(loc2) is expected


class TestStringRepresentations:
    """Tests for `__str__` and `__repr__` methods."""

    def test_str_and_repr(self) -> None:
        """Test that __str__ and __repr__ return expected format."""
        loc = Location(3, -2)
        expected_str = "(3, -2)"
        assert str(loc) == expected_str
        assert repr(loc) == expected_str


class TestHashEquality:
    """Tests for equality and hashing behavior."""

    def test_hash_and_equality(self) -> None:
        """Test that Locations with same coords are equal and have same hash."""
        loc1 = Location(1, 2)
        loc2 = Location(1, 2)
        loc3 = Location(2, 1)
        assert loc1 == loc2
        assert hash(loc1) == hash(loc2)
        assert loc1 != loc3


class TestOrderingComparisons:
    """Tests for ordering comparison methods: <, >, <=, >=."""

    def test_lt_gt_le_ge(self) -> None:
        """Test that comparison operators compare first by x, then by y when x is equal."""
        a = Location(0, 0)
        b = Location(1, 0)
        c = Location(1, 1)
        assert a < b
        assert c > b
        assert a <= b
        assert c >= b
        assert a <= Location(0, 0)
        assert b >= Location(1, 0)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: event.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'event.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import game_pb2 as game__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0b\x65vent.proto\x12\x05\x61\x65gis\x1a\ngame.proto\"\x90\x02\n\x05\x45vent\x12*\n\x0cgames_header\x18\x01 \x01(\x0b\x32\x12.aegis.GamesHeaderH\x00\x12(\n\x0bgame_header\x18\x02 \x01(\x0b\x32\x11.aegis.GameHeaderH\x00\x12\x1d\n\x05round\x18\x03 \x01(\x0b\x32\x0c.aegis.RoundH\x00\x12(\n\x0bgame_footer\x18\x04 \x01(\x0b\x32\x11.aegis.GameFooterH\x00\x12*\n\x0cgames_footer\x18\x05 \x01(\x0b\x32\x12.aegis.GamesFooterH\x00\x12\x33\n\x11\x64rone_scan_update\x18\x06 \x01(\x0b\x32\x16.aegis.DroneScanUpdateH\x00\x42\x07\n\x05\x65ventb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'event_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_EVENT']._serialized_start=35
  _globals['_EVENT']._serialized_end=307
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: world_object.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'world_object.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12world_object.proto\x12\x05\x61\x65gis\"K\n\x08Survivor\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0e\n\x06health\x18\x02 \x01(\x05\x12#\n\x05state\x18\x03 \x01(\x0e\x32\x14.aegis.SurvivorState\"F\n\x06Rubble\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x17\n\x0f\x65nergy_required\x18\x02 \x01(\x05\x12\x17\n\x0f\x61gents_required\x18\x03 \x01(\x05\"]\n\x0bWorldObject\x12#\n\x08survivor\x18\x01 \x01(\x0b\x32\x0f.aegis.SurvivorH\x00\x12\x1f\n\x06rubble\x18\x02 \x01(\x0b\x32\r.aegis.RubbleH\x00\x42\x08\n\x06object*$\n\rSurvivorState\x12\t\n\x05\x41LIVE\x10\x00\x12\x08\n\x04\x44\x45\x41\x44\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'world_object_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SURVIVORSTATE']._serialized_start=273
  _globals['_SURVIVORSTATE']._serialized_end=309
  _globals['_SURVIVOR']._serialized_start=29
  _globals['_SURVIVOR']._serialized_end=104
  _globals['_RUBBLE']._serialized_start=106
  _globals['_RUBBLE']._serialized_end=176
  _globals['_WORLDOBJECT']._serialized_start=178
  _globals['_WORLDOBJECT']._serialized_end=271
# @@protoc_insertion_point(module_scope)
